﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using Lib.Web.Mvc.JQuery.JqGrid;
using Microsoft.Practices.Unity;
using Northwind.Repositories;
using Northwind.Repositories.Entity;
using Lib.Web.Mvc.Unity;

namespace jqGrid
{
    public class MvcApplication : System.Web.HttpApplication
    {
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");
            routes.MapRoute("Default", "{controller}/{action}", new { controller = "Home", action = "Formatting" });
        }

        public void RegisterDependencyResolver()
        {
            IUnityContainer container = new UnityContainer()
                .RegisterType<ICategoriesRepository, CategoriesRepository>()
                .RegisterType<IEmployeesRepository, EmployeesRepository>()
                .RegisterType<IOrdersRepository, OrdersRepository>()
                .RegisterType<IProductsRepository, ProductsRepository>()
                .RegisterType<ISuppliersRepository, SuppliersRepository>()
                .RegisterType<ITerritoriesRepository, TerritoriesRepository>()
                //Multiply registered types must be named
                .RegisterType<IModelBinderProvider, UnityModelBinderProvider>("UnityModelBinderProvider", new ContainerControlledLifetimeManager());

            DependencyResolver.SetResolver(new UnityDependencyResolver(container));
        }

        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();

            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);
            RegisterDependencyResolver();

            JqGridRequest.ParameterNames = new JqGridParametersNames() { PagesCount = "npage" };
            JqGridResponse.JsonReader.RepeatItems = false;
            JqGridResponse.JsonReader.RecordId = "Id";
            JqGridResponse.JsonReader.SubgridReader.RepeatItems = false;
        }
    }
}