﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel;
using Lib.Web.Mvc.JQuery.JqGrid.DataAnnotations;
using Lib.Web.Mvc.JQuery.JqGrid;
using System.ComponentModel.DataAnnotations;
using Northwind.Model;
using Lib.Web.Mvc.JQuery.JqGrid.Constants;

namespace jqGrid.Models
{
    public class ProductCellEditableViewModel
    {
        #region Properties
        [JqGridColumnEditable(false)]
        public int Id { get; set; }

        [JqGridColumnEditable(true, EditType = JqGridColumnEditTypes.Text)]
        [StringLength(40)]
        [Required]
        public string Name { get; set; }

        [JqGridColumnSortable(true, Index = "SupplierId")]
        [JqGridColumnFormatter("$.supplierFormatter")]
        [JqGridColumnEditable(false)]
        public string Supplier { get; set; }

        [JqGridColumnSortable(true, Index = "CategoryId")]
        [JqGridColumnEditable(true, "Categories", "Home", EditType = JqGridColumnEditTypes.Select)]
        [Required]
        public string Category { get; set; }

        [DisplayName("Quantity Per Unit")]
        [JqGridColumnEditable(true, EditType = JqGridColumnEditTypes.Text)]
        [StringLength(20)]
        [Required]
        public string QuantityPerUnit { get; set; }

        [DisplayName("Unit Price")]
        [JqGridColumnFormatter(JqGridColumnPredefinedFormatters.Currency, DecimalPlaces = 2, DecimalSeparator = ".", Prefix = "$")]
        [JqGridColumnEditable(true, EditType = JqGridColumnEditTypes.Text)]
        [Range(0, Int32.MaxValue)]
        [Required]
        public decimal? UnitPrice { get; set; }

        [DisplayName("Units In Stock")]
        [JqGridColumnFormatter("$.unitsInStockFormatter")]
        [JqGridColumnEditable(false)]
        public short? UnitsInStock { get; set; }
        #endregion

        #region Constructor
        public ProductCellEditableViewModel()
        { }

        public ProductCellEditableViewModel(Product product)
        {
            this.Id = product.Id;
            this.Name = product.Name;
            this.Supplier = String.Format("[{0}] {1}", product.SupplierId, product.Supplier.Name);
            this.Category = product.Category.Name;
            this.QuantityPerUnit = product.QuantityPerUnit;
            this.UnitPrice = product.UnitPrice;
            this.UnitsInStock = product.UnitsInStock;
        }
        #endregion
    }
}