﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Northwind.Repositories;
using Northwind.Repositories.Entity;
using System.Globalization;

namespace jqGrid.Models
{
    public class ProductEditableViewModelBinder : DefaultModelBinder
    {
        #region Fields
        ISuppliersRepository _suppliersRepository;
        ICategoriesRepository _categoriesRepository;
        #endregion

        #region Constructor
        public ProductEditableViewModelBinder()
            : this(new SuppliersRepository(), new CategoriesRepository())
        {
        }

        public ProductEditableViewModelBinder(ISuppliersRepository suppliersRepository, ICategoriesRepository categoriesRepository)
        {
            _suppliersRepository = suppliersRepository;
            _categoriesRepository = categoriesRepository;
        }
        #endregion

        #region Methods
        public override object BindModel(ControllerContext controllerContext, ModelBindingContext bindingContext)
        {
            ProductInlineEditableViewModel model = new ProductInlineEditableViewModel();

            string id = bindingContext.ValueProvider.GetValue("id").AttemptedValue;
            if (id != "_empty")
                model.Id = Convert.ToInt32(id);
            else
                model.Id = -1;

            model.Name = Convert.ToString(bindingContext.ValueProvider.GetValue("Name").AttemptedValue);
            model.SupplierId = Convert.ToInt32(bindingContext.ValueProvider.GetValue("Supplier").AttemptedValue);
            model.Supplier = _suppliersRepository.FindByKey(model.SupplierId).Name;
            model.CategoryId = Convert.ToInt32(bindingContext.ValueProvider.GetValue("Category").AttemptedValue);
            model.Category = _categoriesRepository.FindByKey(model.CategoryId).Name;
            model.QuantityPerUnit = Convert.ToString(bindingContext.ValueProvider.GetValue("QuantityPerUnit").AttemptedValue);
            model.UnitPrice = Convert.ToDecimal(bindingContext.ValueProvider.GetValue("UnitPrice").AttemptedValue.Replace(".", CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator));
            model.UnitsInStock = Convert.ToInt16(bindingContext.ValueProvider.GetValue("UnitsInStock").AttemptedValue);
            
            return model;
        }
        #endregion
    }
}