﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel;
using Lib.Web.Mvc.JQuery.JqGrid.DataAnnotations;
using Lib.Web.Mvc.JQuery.JqGrid;
using System.ComponentModel.DataAnnotations;
using Northwind.Model;
using Lib.Web.Mvc.JQuery.JqGrid.Constants;

namespace jqGrid.Models
{
    public class ProductJQueryUIViewModel
    {
        #region Properties
        [ScaffoldColumn(false)]
        public int Id { get; set; }

        [JqGridColumnSearchable(true, "ProductsNames", "Home", SearchType = JqGridColumnSearchTypes.JQueryUIAutocomplete, MinLength = 2)]
        public string Name { get; set; }

        [JqGridColumnSortable(true, Index = "SupplierId")]
        [JqGridColumnFormatter("$.supplierFormatter")]
        [JqGridColumnSearchable(true, "Suppliers", "Home", SearchType = JqGridColumnSearchTypes.Select, SearchOperators = JqGridSearchOperators.Eq)]
        public string Supplier { get; set; }

        [JqGridColumnSearchable(true, "Categories", "Home", SearchType = JqGridColumnSearchTypes.Select, SearchOperators = JqGridSearchOperators.Eq)]
        public string Category { get; set; }

        [DisplayName("Quantity Per Unit")]
        [JqGridColumnLayout(Alignment = JqGridAlignments.Center)]
        [JqGridColumnSearchable(false)]
        public string QuantityPerUnit { get; set; }

        [DisplayName("Unit Price")]
        [JqGridColumnLayout(Alignment = JqGridAlignments.Center)]
        [JqGridColumnFormatter(JqGridColumnPredefinedFormatters.Currency, DecimalPlaces = 2, DecimalSeparator = ".", Prefix = "$")]
        [JqGridColumnSearchable(false)]
        public decimal? UnitPrice { get; set; }

        [Required]
        [Range(0, 32767)]
        [DisplayName("Units In Stock")]
        [JqGridColumnSearchable(false)]
        [JqGridColumnLayout(Alignment = JqGridAlignments.Center)]
        [JqGridColumnEditable(true, EditType = JqGridColumnEditTypes.JQueryUISpinner)]
        [JqGridColumnFormatter("$.unitsInStockFormatter", UnFormatter = "$.unitsInStockUnFormatter")]
        public short? UnitsInStock { get; set; }
        #endregion

        #region Constructor
        public ProductJQueryUIViewModel()
        { }

        public ProductJQueryUIViewModel(Product product)
        {
            this.Id = product.Id;
            this.Name = product.Name;
            this.Supplier = String.Format("[{0}] {1}", product.SupplierId, product.Supplier.Name);
            this.Category = product.Category.Name;
            this.QuantityPerUnit = product.QuantityPerUnit;
            this.UnitPrice = product.UnitPrice;
            this.UnitsInStock = product.UnitsInStock;
        }
        #endregion
    }
}